from lightgbm import LGBMClassifier

from player_performance_ratings import ColumnNames, MatchPredictor, SKLearnClassifierWrapper, PredictColumnNames, \
    LogLossScorer
from player_performance_ratings.examples.utils import load_lol_data
from player_performance_ratings.ratings.enums import RatingColumnNames
from player_performance_ratings.ratings.time_weight_ratings import BayesianTimeWeightedRating
from player_performance_ratings.scorer.score import OrdinalLossScorer

column_names_kpm = ColumnNames(
    team_id='teamname',
    match_id='gameid',
    start_date="date",
    player_id="playername",
    performance='kills_per_minute',
    league='league',
    position='position',
)

column_names_kills = ColumnNames(
    team_id='teamname',
    match_id='gameid',
    start_date="date",
    player_id="playername",
    performance='kills',
    league='league',
    position='position'
)

df = load_lol_data()
df = df.sort_values(by=['date', 'gameid', 'teamname', "playername"])
df['__target'] = df['kills']

df['kills_per_minute'] = df['kills'] / df['gamelength'] * 60

df = (
    df.loc[lambda x: x.position != 'team']
    .assign(team_count=df.groupby('gameid')['teamname'].transform('nunique'))
    .loc[lambda x: x.team_count == 2]
)

time_weighed_rating_kills_per_minute = BayesianTimeWeightedRating()
time_weighed_rating_kills = BayesianTimeWeightedRating()

match_predictor = MatchPredictor(
    rating_generators=[time_weighed_rating_kills_per_minute, time_weighed_rating_kills],
    column_names=[column_names_kpm, column_names_kills],
    predictor=SKLearnClassifierWrapper(
        model=LGBMClassifier(),
        features=[
            RatingColumnNames.TIME_WEIGHTED_RATING + "0",
            RatingColumnNames.TIME_WEIGHTED_RATING + "1",
            RatingColumnNames.TIME_WEIGHTED_RATING_LIKELIHOOD_RATIO + "0",
            RatingColumnNames.TIME_WEIGHTED_RATING_LIKELIHOOD_RATIO + "1",
            RatingColumnNames.TIME_WEIGHTED_RATING_EVIDENCE + "0",
            RatingColumnNames.TIME_WEIGHTED_RATING_EVIDENCE + "1",
                  column_names_kills.position
        ],
        target=PredictColumnNames.TARGET,
    )
)

df_predictions = match_predictor.generate_historical(df=df)

for idx, kills in enumerate(match_predictor.predictor.model.classes_):
    print(df_predictions.iloc[500]['playername'], kills, df_predictions.iloc[500][match_predictor.predictor.pred_column][idx])

print(match_predictor.predictor.model.feature_importances_)

scorer = OrdinalLossScorer(
    pred_column=match_predictor.predictor.pred_column,
)

print(scorer.score(df_predictions, classes_=match_predictor.predictor.model.classes_))




from dataclasses import dataclass
from typing import List, Optional, Union
from dataclasses_json import DataClassJsonMixin


@dataclass
class ColumnNames:
    team_id: str
    match_id: str
    start_date: str
    player_id: str
    performance: str
    league: Optional[str] = None
    position: Optional[str] = None
    participation_weight: str = None
    team_players_percentage_playing_time: str = None
    rating_update_id: str = None

    def __post_init__(self):
        if self.rating_update_id is None:
            self.rating_update_id = self.match_id


@dataclass
class StartRatingParameters:
    start_league_ratings: Optional[dict[str, float]] = None
    league_quantile: float = 0.2
    team_rating_subtract: float = 80
    team_weight: float = 0.2


@dataclass
class MatchPerformance:
    performance_value: float
    participation_weight: float


@dataclass
class PlayerRating(DataClassJsonMixin):
    id: str
    rating_value: float
    name: Optional[str] = None
    games_played: Union[float, int] = 0
    last_match_day_number: int = None
    confidence_sum: float = 0
    prev_rating_changes: List[float] = None


@dataclass
class Team:
    id: str
    player_ids: list[str]
    last_match_day_number: int
    name: Optional[str] = None


@dataclass
class TeamRating(DataClassJsonMixin):
    id: str
    name: str
    rating_value: float
    players: list[PlayerRating]
    games_played: int = 0
    last_match_day_number: int = None


@dataclass
class PreMatchPlayerRating:
    id: str
    rating_value: float
    games_played: int
    league: Optional[str]
    position: Optional[str]
    match_performance: MatchPerformance


@dataclass
class PreMatchTeamRating:
    id: str
    players: list[PreMatchPlayerRating]
    rating_value: float
    league: Optional[str]


@dataclass
class PreMatchRating:
    id: str
    teams: list[PreMatchTeamRating]
    day_number: int


@dataclass
class PlayerRatingChange:
    id: str
    day_number: int
    league: Optional[str]
    participation_weight: float
    predicted_performance: float
    performance: float
    pre_match_rating_value: float
    rating_change_value: float


@dataclass
class TeamRatingChange:
    id: str
    players: list[PlayerRatingChange]
    predicted_performance: float
    performance: float
    pre_match_rating_value: float
    rating_change_value: float
    league: Optional[str]


@dataclass
class PostMatchTeamRatingChange:
    id: str
    players: list[PlayerRatingChange]
    rating_value: float
    predicted_performance: float


@dataclass
class PostMatchRatingChange:
    id: str
    teams: list[PostMatchTeamRatingChange]


@dataclass
class MatchRating:
    id: str
    pre_match_rating: PreMatchRating
    post_match_rating: PostMatchRatingChange


@dataclass
class MatchRatings:
    pre_match_team_rating_values: list[float]
    pre_match_player_rating_values: list[float]
    pre_match_opponent_rating_values: list[float]
    player_rating_changes: list[float]
    player_leagues: list[str]
    team_opponent_leagues: list[str]
    match_ids: list[str]


@dataclass
class MatchPlayer:
    id: str
    performance: MatchPerformance
    league: Optional[str] = None
    position: Optional[str] = None


@dataclass
class MatchTeam:
    id: str
    players: list[MatchPlayer]
    league: str = None


@dataclass
class Match:
    id: str
    update_id: str
    teams: List[MatchTeam]
    day_number: int
    league: str = None
